@@ -17,56 +17,107 @@ import fnmatch
 
 class AddGadget(object):
 
-	def __init__(self, configpath, verbose=False):
+	def __init__(self, configpath, pathname=None, verbose=False, sh=False, enable=False):
 		self.configpath = configpath
+		self.sh = sh
+		self.enable = enable
+		self.pathname = pathname
 		self.verbose = verbose
+
+
+		#print("AddGadget: sh: %s" % (sh), file=sys.stderr)
+		#print("AddGadget: verbose: %s" % (verbose), file=sys.stderr)
 		# self.verbose = True
 
 	def vprint(self, s):
 		if self.verbose:
 			print(s, file=sys.stderr)
 
+	def echo(self, path, s):
+		self.vprint("echo: %s \"%s\"" % (s, path))
+		print("echo \"%s\" > \"%s\"" % (s.strip(), path), file=sys.stdout)
+
+	def echobin(self, path, binarray):
+		bytes = [ "\\\\x%02x" % (b) for b in binarray ]
+		s = "".join(map(str, bytes))
+		print("echo -ne \"%s\" > \"%s\"" % (s, path), file=sys.stdout)
+
 	def write_str(self, path, s):
-		self.vprint("write_str: %s \"%s\"" % (path, s.strip()))
+		self.vprint("write_str: %s \"%s\"" % (s.strip(), path))
 		try:
 			f = open(path, "a")
 			f.writelines(s)
 			f.close()
+		except (FileNotFoundError):
+			self.vprint("%s %s FILE NOT FOUND ERROR" % (path, s.strip()))
+			exit(1)
 		except (PermissionError):
-			print("%s %s PERMISSION DENIED" % (path, s.strip()), file=sys.stderr)
+			self.vprint("%s %s PERMISSION DENIED" % (path, s.strip()))
+			exit(1)
 
 	def write_bytes(self, path, bytes):
 		self.vprint("write_bytes: %s \"%s\"" % (path, bytes))
-		print("write_bytes: %s \"%s\"" % (path, bytes), file=sys.stderr)
-		intarray = [ int(b,0) for b in bytes]
+		# print("write_bytes: %s \"%s\"" % (path, bytes), file=sys.stderr)
+		intarray = [int(b, 0) for b in bytes]
 		binarray = bytearray(intarray)
-		print("write_bytes: %s" % (binarray), file=sys.stderr)
+		if self.sh:
+			self.echobin(path, binarray)
+			return
+		# print("write_bytes: %s" % (binarray), file=sys.stderr)
 		try:
 			f = open(path, "ab")
 			f.write(binarray)
 			f.close()
+		except (FileNotFoundError):
+			self.vprint("%s %s FILE NOT FOUND ERROR" % (path, s.strip()))
+			exit(1)
 		except (PermissionError):
-			print("%s %s PERMISSION DENIED" % (path, s.strip()), file=sys.stderr)
+			self.vprint("%s %s PERMISSION DENIED" % (path, s.strip()))
+			exit(1)
+
+	def write_8bytes(self, path, s):
+		self.vprint("write_bytes: %s \"%s\"" % (path, bytes))
+		while len(s) < 8: s += '\0'
+		binarray = s.encode()
+		if self.sh:
+			self.echobin(path, binarray)
+			return
+		# print("write_8bytes: %s" % (binarray), file=sys.stderr)
+		try:
+			f = open(path, "ab")
+			f.write(binarray)
+			f.close()
+		except (FileNotFoundError):
+			self.vprint("%s %s FILE NOT FOUND ERROR" % (path, s.strip()))
+			exit(1)
+		except (PermissionError):
+			self.vprint("%s %s PERMISSION DENIED" % (path, s.strip()))
+			exit(1)
 
 	def makedirs(self, lpath, existsok=False):
 		self.vprint("makedirs: %s" % (lpath))
+		if self.sh:
+			print("mkdir -p \"%s\"" % (lpath), file=sys.stdout)
+			return
 		try:
 			os.makedirs(lpath)
 		except (FileExistsError):
 			if existsok: return
-			print("makedirs: %s FileExistsError" % (lpath), file=sys.stderr)
+			self.vprint("makedirs: %s FileExistsError" % (lpath))
 			exit(1)
 
-
 	def symlink(self, src, target):
+		if self.sh:
+			print("ln -s \"%s\" \"%s\"" % (src, target), file=sys.stdout)
+			return
 		self.vprint("symlink: %s -> %s" % (target, src))
 		os.symlink(src, target)
 
 	def hex_or_str(self, v):
 		if isinstance(v, (int)):
-			return "0x%02x\n" % v
+			return "0x%02x" % v
 		else:
-			return "%s\n" % v
+			return "%s" % v
 
 	# create_strings
 	# For each language create a stings/lang directory containing
@@ -85,29 +136,36 @@ class AddGadget(object):
 			self.makedirs(spath)
 			string_dict = string_dicts[lang]
 			for s in string_dict:
-				self.write_str("%s/%s" % (spath, s), "%s\n" % string_dict[s])
+				if self.sh:
+					self.echo("%s/%s" % (spath, s), "${%s:-%s}" % (s, string_dict[s]))
+				else:
+					self.write_str("%s/%s" % (spath, s), string_dict[s])
 
 	# add_attrs
 	# Add misc attributes to a path, convert ints to hex strings,
 	# ignoring dictionary entries with values that are not int or str.
 	#
 	def add_attrs(self, path, dict, exclude=[]):
-		# print("add_attrs: path: %s dict: %s" % (path, dict), file=sys.stderr)
+		#print("add_attrs: path: %s dict: %s" % (path, dict), file=sys.stderr)
 		exclude.append("#*")
 		for a in dict:
 			if any(fnmatch.fnmatch(a, pattern) for pattern in exclude):
 				continue
-			# print("add_attrs: path: %s %s type: %s" % (path, a, type(dict[a])), file=sys.stderr)
+			#print("add_attrs: path: %s %s type: %s" % (path, a, type(dict[a])), file=sys.stderr)
 			if isinstance(dict[a], (int, str)):
-				self.write_str("%s/%s" % (path, a), self.hex_or_str(dict[a]))
+				if self.sh:
+					self.echo("%s/%s" % (path, a), "${%s:-%s}" % (a, self.hex_or_str(dict[a])))
+				else:
+					self.write_str("%s/%s" % (path, a), "%s\n" % (self.hex_or_str(dict[a])))
 
 	# create_device_os_desc
-	# Create the Gadget Device os_descs directory, with symlink to
+	# Create the Gadget Device os_desc directory, with symlink to
 	# configuration.
 	def create_device_os_desc(self, path, device_definition):
 
+		# print('create_device_os_desc: %s' % (device_definition))
 		try:
-			os_descs_dict = device_definition['os_descs']
+			os_descs_dict = device_definition['os_desc']
 		except (KeyError):
 			return
 
@@ -115,8 +173,14 @@ class AddGadget(object):
 		if os_descs_dict is None:
 			return
 
+		if 'use' not in os_descs_dict:
+			return
+
+		if os_descs_dict['use'] != "1":
+			return
+
 		lpath = "%s/os_desc" % (path)
-		self.makedirs(lpath)
+		self.makedirs(lpath, existsok=True)
 
 		# See if we have separate config_id and config_name, use
 		# to compose config name to build src and target paths.
@@ -151,8 +215,6 @@ class AddGadget(object):
 			if any(fnmatch.fnmatch(function_name, pattern) for pattern in exclude):
 				continue
 
-			#print("create_functions: %s type: %s %s" % (function_name, type(functions_dict[function_name]), functions_dict[function_name]), file=sys.stderr)
-
 			function_path = "%s/%s" % (functions_path, function_name)
 			function_dict = functions_dict[function_name]
 
@@ -162,24 +224,28 @@ class AddGadget(object):
 			# mass storage has sub lun.0...lun.N directories
 			for l in function_dict:
 				if not fnmatch.fnmatch(l, 'lun.*'): continue
-				print("****\ncreate_functions: %s" % (l), file=sys.stderr)
+				# print("****\ncreate_functions: %s" % (l), file=sys.stderr)
 				self.create_subfunctions("%s/%s" % (function_path, l), function_dict[l])
 
-			# os_descs is optional, may not be present
-			if 'os_descs' in function_dict:
-				function_os_descs = function_dict['os_descs']
+			# os_desc is optional, may not be present
+			if 'os_desc' in function_dict:
+				function_os_descs = function_dict['os_desc']
 				for interface in function_os_descs:
+					interface_ids = function_os_descs[interface]
 					ipath = "%s/os_desc/%s" % (function_path, interface)
 					# print("create_functions: ipath: %s" % (ipath), file=sys.stderr)
+					# print("create_functions: %s" % (interface_ids), file=sys.stderr)
+
+					self.makedirs("%s/os_desc" % (function_path), existsok=True)
+					self.makedirs(ipath, existsok=True)
+					for id in ['compatible_id', 'sub_compatible_id']:
+						self.write_8bytes("%s/%s" % (ipath, id), interface_ids[id])
 
 			# report_descs is optional, typical hid only
 			if 'report_desc' in function_dict:
 				rpath = "%s/report_desc" % (function_path)
 				self.write_bytes(rpath, function_dict['report_desc'])
 
-
-
-
 	# create_configs
 	# Create the Gadget Device Configurations
 	def create_configs(self, path, configs_dict):
@@ -214,6 +280,14 @@ class AddGadget(object):
 	# Create a Gadget Device definition from saved json configuratiion
 	#
 	def add_device_json(self, device_definition, device_name=None):
+
+		self.vprint("add_device_json: %s" % (device_definition))
+		if self.sh:
+			print("#!/bin/sh", file=sys.stdout)
+			print("# Created from %s\n" % (self.pathname), file=sys.stdout)
+			print("# Usage: manufacturer=$1 scriptname.sh", file=sys.stdout)
+			print("", file=sys.stdout)
+
 		device_path = "%s/%s" % (self.configpath, device_name)
 		self.makedirs(device_path)
 
@@ -228,3 +302,6 @@ class AddGadget(object):
 		self.create_configs(device_path, device_definition['configs'])
 		self.create_device_os_desc(device_path, device_definition)
 
+		if self.sh and self.enable:
+			print("\nbasename /sys/class/udc/* > %s/UDC" % (device_path))
+
